function  Xstml=stml_BCCB(P,P_center,B,sigma_w,sigma_e)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the STML solution for the linear system A_o x =b where A_o is a
% block circulant matrix given by a corresponding point spread function
% Based on the paper
% Amir Beck and Yonina Eldar, "Structured Total Maximum Likelihood:An Alternative to Structured Total Least-Squares"
% -----------------------------------------------------------------------
% Copyright (2009): Amir Beck and Yonina Eldar
% 
% distributed under the terms of 
% the GNU General Public License 2.0.
% 
% Permission to use, copy, modify, and distribute this software for
% any purpose without fee is hereby granted, provided that this entire
% notice is included in all copies of any software which is or includes
% a copy or modification of this software and in all copies of the
% supporting documentation for such software.
% This software is being provided "as is", without any express or
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose."
%-------------------------------------------------------------
% INPUT
%
% P ...................................... the observed (noisy) PSF
% P_center .................... the center of the PSF P
% B ...................................... the observed (noisy)  image
% sigma_w ......................  standard deviation of the righthand side vector
% sigma_e ......................  standard deviation of the structure components.
%
% OUTPUT
% 
% X.......................................  The (global) solution of the
%                                             STML problem  corresponding
%                                             to the linear system A_o x =
%                                             b_o. X is the rectangular
%                                             form of x

[m,n]=size(B);
N=m*n;
Pbig = zeros(size(B));
[m1,n1]=size(P);
Pbig(1:m1, 1:n1) = P;
% a is the vetor of eigenvalues of the matrix A
a=vect(fft2(circshift(Pbig,1-P_center)));
min(abs(a))
% b is the vectorized two-dimensional FFT transform of the observed image B
b=vect(1/sqrt(N)*fft2(B));

% setting the parameters c,d for the 1D solver
c=N*sigma_e^2;
d=sigma_w^2;

% Finding the FFT components of the solution
% computing the transform  of the solution using the 1D solver
x_tilde=solve_1d(a,b,c,d,1e-8);

% Constructing the solution using an inverse DFT transform
Xstml=sqrt(N)*real(ifft2(reshape(x_tilde,[m,n])));

function x=vect(X)
x=X(:);

